package com.filemaker.client;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.JFormattedTextField;
import javax.swing.JSpinner;
import javax.swing.SpinnerDateModel;
import javax.swing.SpinnerModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.DateFormatter;
import javax.swing.text.DefaultFormatterFactory;

import com.ulcjava.base.client.UIComponent;
import com.ulcjava.base.client.util.IChangedListener;
import com.ulcjava.base.client.util.IEnablingSource;
import com.ulcjava.base.client.util.IEnablingTarget;
import com.ulcjava.base.client.util.IHasChangedSource;

public class UIDateSpinner extends UIComponent 
	implements ChangeListener, IEnablingSource, IHasChangedSource, IEnablingTarget {
	 
	private List<IEnablingTarget> enablingTargets = new ArrayList<IEnablingTarget>();
	private Vector<IChangedListener> changedListeners = new Vector<IChangedListener>() ;
		 
	protected class BasicSpinner extends JSpinner {
		public BasicSpinner() {
		}

		public BasicSpinner(SpinnerModel model) {
			super(model);
		    // Get the date formatter
		    JFormattedTextField tf =
		        ((JSpinner.DefaultEditor)getEditor()).getTextField();
		    DefaultFormatterFactory factory =
		        (DefaultFormatterFactory)tf.getFormatterFactory();
		    DateFormatter formatter = (DateFormatter)factory.getDefaultFormatter();
		    
		    // Change the date format to only show the hours
		    formatter.setFormat(new SimpleDateFormat("hh:00 a"));
		}
	}

	@Override
	protected Object createBasicObject(Object[] o) {
		
		Date value = (Date) o[0];

	    // Create a calendar object and initialize to a particular hour if desired
	    Calendar calendar = new GregorianCalendar();
	    calendar.set(Calendar.HOUR_OF_DAY, 13); // 1pm
	    
	    SpinnerDateModel model = new SpinnerDateModel(
	    		 calendar.getTime(), null, null, Calendar.MINUTE);
	    
	    
//		SpinnerDateModel model = new SpinnerDateModel();
//		if (value != null)
//			model.setValue(value);
	
		return new BasicSpinner(model);
	}

	public BasicSpinner getBasicSpinner() {
		return (BasicSpinner) getBasicObject();
	}
		    
	@Override
	protected void postInitializeState() {
		super.postInitializeState();		
		getModel().addChangeListener(this);
	}
	
	public SpinnerDateModel getModel() {
		return (SpinnerDateModel) getBasicSpinner().getModel() ;
	}

	public void addTarget(IEnablingTarget ienablingtarget) {
		enablingTargets.add(ienablingtarget);
		ienablingtarget.setEnabled(isEnabling());
	}
		    
	public boolean isEnabling() {
		return getBasicSpinner().getValue() != null;
	}

	public void removeTarget(IEnablingTarget ienablingtarget) {
		enablingTargets.remove(ienablingtarget);
	}

	private void updateEnablingTargets() {
		Iterator<IEnablingTarget> iterator = enablingTargets.iterator();
		while (iterator.hasNext()) {
			IEnablingTarget ienablingtarget = iterator.next();
			ienablingtarget.setEnabled(isEnabling());
		}
	}

	public void addHasChangedListener(IChangedListener arg0) {
		changedListeners.add(arg0);
	}

	public void removeHasChangedListener(IChangedListener arg0) {
		changedListeners.remove(arg0);
	}
	
	public void stateChanged(ChangeEvent e) {
		updateEnablingTargets();
	    updateStateULC("value", getValue());
	    fireValueChangedULC();
	    for(IChangedListener l : changedListeners)
	    	l.changed() ;
	}
	
	  protected void fireValueChangedULC() {
			fireMandatoryEventULC("valueChanged", "valueChanged", EMPTY_ARGUMENTS);
	  }
	
	public Object getValue() {
		return getBasicSpinner().getValue();		
	}
	 
	 public void setValue(Date date) {
		 getModel().setValue(date);
	 }	
}

