package com.filemaker.server;

import com.ulcjava.base.application.ULCComponent;
import com.ulcjava.base.application.ULCProxy;
import com.ulcjava.base.application.enabler.IEnabler;
import com.ulcjava.base.application.enabler.IHasChangedSource;
import com.ulcjava.base.application.event.IValueChangedListener;
import com.ulcjava.base.application.event.ValueChangedEvent;
import com.ulcjava.base.server.IDispatcher;

public class ULCSpinner extends ULCComponent implements IEnabler, IHasChangedSource {
	public final static String VALUECHANGED_CATEGORY = "valueChanged" ;

	private static final String VALUE_PROP = "value";
	private static final String MINIMUM_PROP = "minimum";
	private static final String MAXIMUM_PROP = "maximum";
	private static final String STEPSIZE_PROP = "stepSize";
	
	private Comparable minimum = null ;
	private Comparable maximum = null ;
	private Number stepSize = null ;
	protected Number value = null ;
	
	protected class ULCSpinnerDispatcher extends ULCComponentDispatcher {
		public void updateValue(Number value) {
			ULCSpinner.this.value = value ;
		}
    }
	
	public ULCSpinner() {
		this(null, null, null, null);
	}
	
	public ULCSpinner(Number value, Comparable min, Comparable max, Number stepSize) {
		this.value = value ;
		this.minimum = min ;
		this.maximum = max ;
		this.stepSize = stepSize ;
	}

	@Override
	protected IDispatcher createDispatcher() {
		return new ULCSpinnerDispatcher();
	}
	
	@Override
	protected String getPropertyPrefix() {
		return "ULCSpinner";
	}

	@Override
	protected String typeString() {
		return "UISpinner";
	}

	public void addValueChangedListener(IValueChangedListener listener) {
    	addListener(VALUECHANGED_CATEGORY,listener);
    }
   
    public void removeValueChangedListener(IValueChangedListener listener) {
    	removeListener(VALUECHANGED_CATEGORY,listener);
    }
    
   @Override
	protected void uploadStateUI() {
	   	super.uploadStateUI();
		createStateUI(new Object[] {value, minimum, maximum, stepSize});
	}
	
	public Number getValue() {
    	return value;
    }

	public Comparable getMinimum() {
		return minimum;
	}

	public Number getStepSize() {
		return stepSize;
	}

	public Comparable getMaximum() {
		return maximum;
	}

	public void setValue(Number value) {
		this.value = (Number) setStateUI(VALUE_PROP, this.value, value);
	}
	
	public void setMaximum(Comparable maximum) {
		this.maximum = (Comparable) setStateUI(MAXIMUM_PROP, this.maximum, maximum);
	}

	public void setMinimum(Comparable minimum) {
		this.minimum = (Comparable) setStateUI(MINIMUM_PROP, this.minimum, minimum);
	}

	public void setStepSize(Number stepSize) {
		this.stepSize = (Number) setStateUI(STEPSIZE_PROP, this.stepSize, stepSize);
	}
}