/**
 * @author Sandro <sandro.pedrazzini@canoo.com>
 */

package com.canoo.ulc.community.datechooser.client.widget.util;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;

public class LocaleAdapter {
    private Locale fLocale;
    private String fCancelButtonText;

    public LocaleAdapter(){
        this(Locale.getDefault(),"Cancel");
    }

    public LocaleAdapter(Locale locale){
        this(locale,"Cancel");
    }

    public LocaleAdapter(Locale locale, String cancelButtonText) {
        fLocale = locale;
        fCancelButtonText = cancelButtonText;
    }

    public Locale getLocale() {
        return fLocale;
    }

    public void setLocale(Locale locale) {
        fLocale = locale;
    }

    public String getCancelButtonText() {
        return fCancelButtonText;
    }

    public void setCancelButtonText(String cancelButtonText) {
        fCancelButtonText = cancelButtonText;
    }

    int localizeHeaderPosition(int i) {
        int firstDayOfWeek = GregorianCalendar.getInstance(fLocale).getFirstDayOfWeek();
        return (i + firstDayOfWeek - 1 + 7) % 7;
    }

    int localizeDayPosition(int i) {
        int firstDayOfWeek = GregorianCalendar.getInstance(fLocale).getFirstDayOfWeek();
        return (i - firstDayOfWeek + 1 + 7) % 7;
    }

    int localizeDayBaseOne(int i) {
        int firstDayOfWeek = GregorianCalendar.getInstance(fLocale).getFirstDayOfWeek();
        int pos = (i - firstDayOfWeek + 1 + 7) % 7;
        return (pos == 0 ? 7 : pos);
    }


    String[] getDaysOfWeek(int size) {
        String[] days = new String[7];
        Calendar calendar = Calendar.getInstance(fLocale);
        calendar.set(2004, 5, 20); //sunday, the first day of the array to deliver
        for (int i = 0; i < days.length; i++) {
        	SimpleDateFormat formatter = new SimpleDateFormat("E", fLocale);
        	String fullDate = formatter.format(calendar.getTime());
            days[i] = fullDate.substring(0, Math.min(size, fullDate.length()));
            calendar.add(Calendar.DATE, 1);
        }
        return days;
    }

    static int localeNormalizedWeekNr(boolean firstWeekIsZero, int week) {
        return (firstWeekIsZero ? week : week - 1);
    }
}
