/**
 * @author Sandro <sandro.pedrazzini@canoo.com>
 */
package com.canoo.ulc.community.datechooser.client.widget.util;


import javax.swing.*;
import javax.swing.plaf.BorderUIResource;
import java.awt.*;
import java.awt.event.*;
import java.text.DateFormat;
import java.text.FieldPosition;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Locale;

/**
 * Layout:
 * <p/>
 * <p/>
 * BoxLayout -Y
 * \_______ Borderlayout  (navigation buttons)
 * \_______ GridLayout    (days headings)
 * \_______ GridLayout    (days)
 * \_______ BorderLayout  (Cancel [today] button)
 */

public class PickDatePanel extends JPanel {
    private static final Icon sMonthBack = new ImageIcon(PickDatePanel.class.getResource("/month-back.png"));
    private static final Icon sMonthForw = new ImageIcon(PickDatePanel.class.getResource("/month-forw.png"));
    private final DaySelection sDaySelection = new DaySelection();

    private Component fCurrentlySelectedDayComponent = null;
    private GregorianCalendar fCurrentlySelectedDate = null;
    private LocaleAdapter fLocaleAdapter;
    private GraphicMemento fGraphicMemento;

    private Date fFrom;
    private Date fTo;

    private final JLabel fMonthAndYear = new JLabel();

    /**
     * 2-dimensional array for 6 weeks of 7 days each.
     */
    private JTextField[][] fDaysInMonth = null;

    /*
    public PickDatePanel(GraphicMemento graphicMemento, LocaleAdapter localeAdapter) {
        this(graphicMemento, localeAdapter, null,null,null);
    } */


    public PickDatePanel(GraphicMemento graphicMemento, LocaleAdapter localeAdapter, Date initialDate, Date from, Date to) {
        fLocaleAdapter = localeAdapter;
        fGraphicMemento = graphicMemento;
        if (initialDate == null)
            fCurrentlySelectedDate = getToday(localeAdapter.getLocale());
        else
            (fCurrentlySelectedDate = new GregorianCalendar(localeAdapter.getLocale())).setTime(initialDate);
        fFrom = from;
        fTo = to;
        fDaysInMonth = createDaysInMonthTextFields();
        initDaysPanel();
    }


    private JTextField[][] createDaysInMonthTextFields() {
        JTextField[][] daysInMonth = new JTextField[6][7];
        for (int i = 0; i < 6; i++) {
            for (int j = 0; j < 7; j++) {
                daysInMonth[i][j] = new JTextField();
            }
        }
        return daysInMonth;
    }


    /**
     * Returns the currently selected date.
     */
    public Date getDate() {
        if (null != fCurrentlySelectedDate)
            return fCurrentlySelectedDate.getTime();
        return null;
    }


    private void initDaysPanel() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        setMinimumSize(new Dimension(fGraphicMemento.getPanelWidth(), fGraphicMemento.getPanelHigh())); //226));
        setMaximumSize(getMinimumSize());
        setPreferredSize(getMinimumSize());
        setBorder(new BorderUIResource.EtchedBorderUIResource());

        JPanel monthNavigationPanel = createNavigationPanel();
        add(monthNavigationPanel);

        // layout the column headings for the days of the week
        JPanel headingsPanel = createDayHeadingsPanel();
        add(Box.createRigidArea(new Dimension(0, fGraphicMemento.getBorder())));
        add(headingsPanel);

        // layout the days of the month
        JPanel daysPanel = createDaysPanel();
        add(daysPanel);

        JPanel cancelPanel = createCancelPanel();
        add(cancelPanel);
        int panelBorder = fGraphicMemento.getBorder();
        setBorder(BorderFactory.createEmptyBorder(panelBorder, panelBorder, panelBorder, panelBorder));
        calculateCalendar();
        setCommonBackground(monthNavigationPanel, headingsPanel, daysPanel, cancelPanel);
    }

    private void setCommonBackground(JPanel monthNavigationPanel, JPanel headingsPanel, JPanel daysPanel, JPanel cancelPanel) {
        if (fGraphicMemento.getDialogBackgroundColor() != null) {
            setBackground(fGraphicMemento.getDialogBackgroundColor());
            monthNavigationPanel.setBackground(fGraphicMemento.getDialogBackgroundColor());
            headingsPanel.setBackground(fGraphicMemento.getDialogBackgroundColor());
            daysPanel.setBackground(fGraphicMemento.getDialogBackgroundColor());
            cancelPanel.setBackground(fGraphicMemento.getDialogBackgroundColor());
        }
    }

    private JPanel createDaysPanel() {
        JPanel daysPanel = new JPanel();
        daysPanel.setLayout(new GridLayout(6, 7));
        for (int horizontalIx = 0; horizontalIx < fDaysInMonth.length; horizontalIx++) {
            for (int verticalIx = 0; verticalIx < fDaysInMonth[horizontalIx].length; verticalIx++) {
                fDaysInMonth[horizontalIx][verticalIx].setBackground(fGraphicMemento.getDisabledCellColor());
                fDaysInMonth[horizontalIx][verticalIx].setEditable(false);
                fDaysInMonth[horizontalIx][verticalIx].setFont(fGraphicMemento.getDaysFont());
                fDaysInMonth[horizontalIx][verticalIx].setHorizontalAlignment(JTextField.RIGHT);
                fDaysInMonth[horizontalIx][verticalIx].setText("");
                //TODO: 1.4 fDaysInMonth[horizontalIx][verticalIx].setFocusable(false);
                fDaysInMonth[horizontalIx][verticalIx].setPreferredSize(fGraphicMemento.getCellDimension());
                fDaysInMonth[horizontalIx][verticalIx].setMaximumSize(fGraphicMemento.getCellDimension());
                fDaysInMonth[horizontalIx][verticalIx].setMinimumSize(fGraphicMemento.getCellDimension());
                daysPanel.add(fDaysInMonth[horizontalIx][verticalIx]);
            }
        }
        daysPanel.setBorder(BorderFactory.createEmptyBorder(fGraphicMemento.getBorder() / 2, 0, fGraphicMemento.getBorder() / 2, 0));
        return daysPanel;
    }

    private JPanel createDayHeadingsPanel() {
        JPanel headingsPanel = new JPanel();
        headingsPanel.setLayout(new GridLayout(1, 7));
        JTextField[] dayHeadings = computeDayHeadings();
        for (int horizontalIx = 0; horizontalIx < dayHeadings.length; horizontalIx++) {
            dayHeadings[horizontalIx].setBackground(fGraphicMemento.getDisabledCellColor());
            dayHeadings[horizontalIx].setEditable(false);
            dayHeadings[horizontalIx].setFont(fGraphicMemento.getDaysFont());
            dayHeadings[horizontalIx].setHorizontalAlignment(JTextField.CENTER);
            //TODO: 1.4 dayHeadings[horizontalIx].setFocusable(false);
            Dimension absoluteDesiredDimension = fGraphicMemento.getCellDimension();
            dayHeadings[horizontalIx].setPreferredSize(absoluteDesiredDimension);
            dayHeadings[horizontalIx].setMaximumSize(absoluteDesiredDimension);
            dayHeadings[horizontalIx].setMinimumSize(absoluteDesiredDimension);
            headingsPanel.add(dayHeadings[horizontalIx]);
        }
        headingsPanel.setBorder(BorderFactory.createEmptyBorder(fGraphicMemento.getBorder() / 2, 0, fGraphicMemento.getBorder() / 2, 0));
        return headingsPanel;
    }

    private JPanel createNavigationPanel() {
        JPanel monthNavigationPanel = new JPanel();
        monthNavigationPanel.setLayout(new BorderLayout());
        final JButton monthBackButton = createNavigationButton(sMonthBack, new BackwardAction());
        final JButton monthForwardButton = createNavigationButton(sMonthForw, new ForwardAction());

        fMonthAndYear.setFont(fGraphicMemento.getLabelsFont());
        fMonthAndYear.setHorizontalAlignment(JTextField.CENTER);
        fMonthAndYear.setText(formatDateText(fCurrentlySelectedDate.getTime(), fLocaleAdapter.getLocale()));
        monthNavigationPanel.add(monthBackButton, BorderLayout.WEST);
        monthNavigationPanel.add(fMonthAndYear, BorderLayout.CENTER);
        monthNavigationPanel.add(monthForwardButton, BorderLayout.EAST);
        monthNavigationPanel.setMinimumSize(new Dimension(fGraphicMemento.getPanelWidth() - 2 * fGraphicMemento.getBorder(), 20));
        monthNavigationPanel.setMaximumSize(monthNavigationPanel.getMinimumSize());
        monthNavigationPanel.setPreferredSize(monthNavigationPanel.getMinimumSize());
        return monthNavigationPanel;
    }


    private JButton createNavigationButton(Icon icon, ActionListener action) {
        final JButton monthShiftButton = new JButton();
        monthShiftButton.setFont(fGraphicMemento.getDaysFont());
        //monthShiftButton.setText(text);
        monthShiftButton.setIcon(icon);
        monthShiftButton.setMargin(fGraphicMemento.getInsets());
        monthShiftButton.setDefaultCapable(false);
        monthShiftButton.addActionListener(action);
        monthShiftButton.setPreferredSize(fGraphicMemento.getCellDimension());
        monthShiftButton.setMaximumSize(fGraphicMemento.getCellDimension());
        monthShiftButton.setMinimumSize(fGraphicMemento.getCellDimension());
        return monthShiftButton;
    }

    private JPanel createCancelPanel() {
        JPanel cancelPanel = new JPanel();
        cancelPanel.setLayout(new BoxLayout(cancelPanel, BoxLayout.X_AXIS));
        final JButton cancelButton = new JButton(fLocaleAdapter.getCancelButtonText());
        cancelButton.setMinimumSize(new Dimension(fGraphicMemento.getButtonWidth(), fGraphicMemento.getButtonHight()));
        cancelButton.setMaximumSize(cancelButton.getMinimumSize());
        cancelButton.setPreferredSize(cancelButton.getMinimumSize());
        cancelButton.setMargin(fGraphicMemento.getInsets());
        cancelButton.setFont(fGraphicMemento.getLabelsFont());
        cancelButton.addActionListener(new CancelAction());
        cancelPanel.add(Box.createRigidArea(new Dimension(fGraphicMemento.getCellWidth(), 0)));
        cancelPanel.add(cancelButton);
        cancelPanel.add(Box.createRigidArea(new Dimension(fGraphicMemento.getCellWidth(), 0)));
        return cancelPanel;
    }


    private JTextField[] computeDayHeadings() {
        JTextField[] dayHeadings = new JTextField[7];
        String[] daysOfWeek = fLocaleAdapter.getDaysOfWeek(2);
        for (int i = 0; i < dayHeadings.length; i++) {
            dayHeadings[i] = new JTextField(daysOfWeek[fLocaleAdapter.localizeHeaderPosition(i)]);
        }
        return dayHeadings;
    }


    class CancelAction implements ActionListener {
        public void actionPerformed(final ActionEvent evt) {
            fCurrentlySelectedDate = null;
            setVisible(false);
        }
    }


    class ForwardAction implements ActionListener {
        public void actionPerformed(final ActionEvent evt) {
            final int day = fCurrentlySelectedDate.get(Calendar.DATE);
            fCurrentlySelectedDate.set(Calendar.DATE, 1);
            fCurrentlySelectedDate.add(Calendar.MONTH, 1);
            fCurrentlySelectedDate.set(Calendar.DATE,
                    Math.min(day, getDaysInMonth(fCurrentlySelectedDate)));
            fMonthAndYear.setText(formatDateText(fCurrentlySelectedDate.getTime(), fLocaleAdapter.getLocale()));
            calculateCalendar();
        }
    }


    class BackwardAction implements ActionListener {
        public void actionPerformed(final ActionEvent evt) {
            final int day = fCurrentlySelectedDate.get(Calendar.DATE);
            fCurrentlySelectedDate.set(Calendar.DATE, 1);
            fCurrentlySelectedDate.add(Calendar.MONTH, -1);
            fCurrentlySelectedDate.set(Calendar.DATE,
                    Math.min(day, getDaysInMonth(fCurrentlySelectedDate)));
            fMonthAndYear.setText(formatDateText(fCurrentlySelectedDate.getTime(), fLocaleAdapter.getLocale()));
            calculateCalendar();
        }
    }

    /**
     * Event handler that sets the currently selected date to the clicked day.
     */
    class DaySelection extends MouseAdapter {
        Color fLastBackground = null;

        public void mouseClicked(final MouseEvent evt) {
            JTextField dayField = (JTextField) evt.getSource();
            if (!"".equals(dayField.getText())) {
                /*if (null != fCurrentlySelectedDayComponent) {
                    fCurrentlySelectedDayComponent.setBackground(fGraphicMemento.getEnabledCellColor());
                }
                dayField.setBackground(fGraphicMemento.getTodayHighlight());  */
                fCurrentlySelectedDayComponent = dayField;
                fCurrentlySelectedDate.set(Calendar.DATE, Integer.parseInt(dayField.getText()));
                setVisible(false);
            }
        }

        public void mouseEntered(MouseEvent evt) {
            JTextField dayField = (JTextField) evt.getSource();
            fLastBackground = dayField.getBackground();
            dayField.setFont(fGraphicMemento.getHighlightFont());
            if (fGraphicMemento.isEnabledCurrentDayView()) {
                try {
                    int value = Integer.parseInt(dayField.getText());
                    fCurrentlySelectedDate.set(Calendar.DATE, value);
                    fMonthAndYear.setText(formatExactDateAsString(fCurrentlySelectedDate.getTime(), fLocaleAdapter.getLocale()));
                } catch (NumberFormatException ignoreIt) {
                }
            }
        }

        public void mouseExited(MouseEvent evt) {
            JTextField dayField = (JTextField) evt.getSource();
            dayField.setFont(fGraphicMemento.getDaysFont());
            fMonthAndYear.setText(formatDateText(fCurrentlySelectedDate.getTime(), fLocaleAdapter.getLocale()));
        }
    }

    public static boolean betweenFromAndTo(Date selectedDate, Date from, Date to){
        if (from != null && from.after(selectedDate)){
            return false;
        }
        if (to != null && to.before(selectedDate)){
            return false;
        }
        return true;
    }


    /**
     * Returns the current date.
     *
     * @return date
     */
    private static GregorianCalendar getToday(Locale locale) {
        final GregorianCalendar gc = new GregorianCalendar(locale);
        gc.set(Calendar.HOUR_OF_DAY, 0);
        gc.set(Calendar.MINUTE, 0);
        gc.set(Calendar.SECOND, 0);
        gc.set(Calendar.MILLISECOND, 0);
        return gc;
    }

    /**
     * Calculates the days of the month.
     */
    private void calculateCalendar() {
        // clear the selected date
        if (null != fCurrentlySelectedDayComponent) {
            fCurrentlySelectedDayComponent.setBackground(fGraphicMemento.getEnabledCellColor());
            fCurrentlySelectedDayComponent = null;
        }

        final GregorianCalendar currentViewedCalendar = getCalendarFirstDayOfCurrentlySelectedMonth();
        int todaySelected = getDayOfCurrentMonthIfToday();
        clearAllDaysToFirstDayOfMonth(currentViewedCalendar);

        // construct the days in the selected month
        int week, dayOfWeek;
        boolean firstWeekIsZero = false;
        int maxDaysInCurrentMonth = getDaysInMonth(currentViewedCalendar);
        do {
            week = currentViewedCalendar.get(Calendar.WEEK_OF_MONTH);
            dayOfWeek = currentViewedCalendar.get(Calendar.DAY_OF_WEEK);
            if (week == 0) {
                firstWeekIsZero = true;
            }
            prepareDayWidget(firstWeekIsZero, week, dayOfWeek, currentViewedCalendar, todaySelected);
            if (currentViewedCalendar.get(Calendar.DATE) >= maxDaysInCurrentMonth)
                break;
            currentViewedCalendar.add(Calendar.DATE, 1);
        } while (currentViewedCalendar.get(Calendar.DATE) <= maxDaysInCurrentMonth);

        // clear all the days after the last day of the month
        int localeFirstWeek = LocaleAdapter.localeNormalizedWeekNr(firstWeekIsZero, week);
        clearAllDaysAfterLastDayOfMonth(localeFirstWeek, fLocaleAdapter.localizeDayBaseOne(dayOfWeek));

        // set the currently selected date
        if (todaySelected != -1) {
            currentViewedCalendar.set(Calendar.DATE, todaySelected);
            fCurrentlySelectedDate = currentViewedCalendar;
        }
    }

    private void prepareDayWidget(boolean firstWeekIsZero, int week, int dayOfWeek, final GregorianCalendar currentViewedCalendar, int todaySelected) {
        JTextField day = fDaysInMonth[LocaleAdapter.localeNormalizedWeekNr(firstWeekIsZero, week)][fLocaleAdapter.localizeDayPosition(dayOfWeek - 1)];
        day.setText(Integer.toString(currentViewedCalendar.get(Calendar.DATE)));
        if (todaySelected == currentViewedCalendar.get(Calendar.DATE)) {
            day.setBackground(fGraphicMemento.getTodayHighlight());
            fCurrentlySelectedDate = currentViewedCalendar;
        } else
            day.setBackground(fGraphicMemento.getEnabledCellColor());
        if (betweenFromAndTo(currentViewedCalendar.getTime(), fFrom, fTo)){
           day.addMouseListener(sDaySelection);
        }else{
            day.setBackground(fGraphicMemento.getDisabledCellColor());
            removeAllMouseListeners(day);
        }
    }

    private void removeAllMouseListeners(JTextField day) {
        MouseListener[] listeners = day.getMouseListeners();
        for (int i=0; i< listeners.length; i++){
            day.removeMouseListener(listeners[i]);
        }
    }

    /**
     * Returns -1 if there is no day to be selected in the current month/year
     */
    private int getDayOfCurrentMonthIfToday() {
        Calendar today = Calendar.getInstance();
        if ((today.get(Calendar.YEAR) == fCurrentlySelectedDate.get(Calendar.YEAR)) &&
                (today.get(Calendar.MONTH) == fCurrentlySelectedDate.get(Calendar.MONTH))) {
            return today.get(Calendar.DATE);
        }
        return -1;
    }

    private GregorianCalendar getCalendarFirstDayOfCurrentlySelectedMonth() {
        GregorianCalendar gregorianCalendar = new GregorianCalendar(fLocaleAdapter.getLocale());
        gregorianCalendar.set(fCurrentlySelectedDate.get(Calendar.YEAR), fCurrentlySelectedDate.get(Calendar.MONTH), 1);
        return gregorianCalendar;
    }


    private void clearAllDaysToFirstDayOfMonth(final GregorianCalendar currentViewedCalendar) {
        int dayOfWeek = fLocaleAdapter.localizeDayBaseOne(currentViewedCalendar.get(Calendar.DAY_OF_WEEK));
        for (int dayToClear = 0; dayToClear < dayOfWeek; dayToClear++) {
            fDaysInMonth[0][dayToClear].setText("");
            fDaysInMonth[0][dayToClear].setBackground(fGraphicMemento.getDisabledCellColor());
        }
    }

    private void clearAllDaysAfterLastDayOfMonth(int localeLastWeek, int dayOfWeek) {
        for (int ww = localeLastWeek; ww < fDaysInMonth.length; ww++) {
            for (int dd = dayOfWeek; dd < fDaysInMonth[ww].length; dd++) {
                fDaysInMonth[ww][dd].setText("");
                fDaysInMonth[ww][dd].setBackground(fGraphicMemento.getDisabledCellColor());
            }
            dayOfWeek = 0;
        }
    }


    /**
     * Calculates the number of days in the specified month.
     */
    private static int getDaysInMonth(final Calendar calendar) {
        int daysInMonth = 0;
        switch (calendar.get(Calendar.MONTH)) {
            case 0:
            case 2:
            case 4:
            case 6:
            case 7:
            case 9:
            case 11:
                daysInMonth = 31;
                break;
            case 3:
            case 5:
            case 8:
            case 10:
                daysInMonth = 30;
                break;
            case 1:
                final int year = calendar.get(Calendar.YEAR);
                daysInMonth = ((((year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0)) ? 29 : 28);
                break;
        }
        return daysInMonth;
    }

    /**
     * Returns a short string representation of the specified date (ex: January, 2004).
     * <p/>
     * TODO: make it customizable through a Command element
     */
    private static String formatDateText(Date date, Locale locale) {
        final DateFormat df = DateFormat.getDateInstance(DateFormat.LONG, locale);

        final StringBuffer mm = new StringBuffer();
        final StringBuffer yy = new StringBuffer();
        final FieldPosition mmfp = new FieldPosition(DateFormat.MONTH_FIELD);
        final FieldPosition yyfp = new FieldPosition(DateFormat.YEAR_FIELD);
        df.format(date, mm, mmfp);
        df.format(date, yy, yyfp);
        return (mm.toString().substring(mmfp.getBeginIndex(), mmfp.getEndIndex()) +
                " " + yy.toString().substring(yyfp.getBeginIndex(), yyfp.getEndIndex()));
    }

    private static String formatExactDateAsString(Date date, Locale locale) {
        return DateFormat.getDateInstance(DateFormat.LONG, locale).format(date);
    }

}
