package com.canoo.ulc.community.datechooser.client.environment;

import com.jgoodies.plaf.LookUtils;
import com.jgoodies.plaf.Options;
import com.jgoodies.plaf.plastic.Plastic3DLookAndFeel;
import com.jgoodies.plaf.plastic.PlasticLookAndFeel;
import com.jgoodies.plaf.plastic.theme.SkyBlue;

import javax.swing.*;

/**
 * This helper class can set different UIManagers.
 *
 * @author Etienne.Studer@canoo.com
 * @see javax.swing.UIManager
 */
public class UIManagerHelper
{
  public static void setJGoodiesLookAndFeelOnWindows()
  {
    if(isWindowsOS())
    {
      JGoodiesHelper.setJGoodiesLookAndFeel();
    }
  }


  private static boolean isWindowsOS()
  {
    String os = System.getProperty("os.name").toLowerCase();
    return os.indexOf("windows") > -1;
  }


  private static void setLookAndFeel(LookAndFeel lookAndFeel)
  {
    try
    {
      UIManager.setLookAndFeel(lookAndFeel);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }


  public static void setSystemLookAndFeel()
  {
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }


  /**
   * Setting the JGoodies L&F is done through this inner static class, in order to avoid that the JGoodies classes are
   * unnecessarily loaded if not needed. This allows for optimization of jar file download to the client-side.
   * <p/>
   * e.g., in the case of JNLP deployment, we might not want to set the JGoodies L&F on Mac systems, and we can hereby
   * avoid that the JGoodies jar file is unnecessarily downloaded to a Mac client.
   */
  private static class JGoodiesHelper
  {
    private static void setJGoodiesLookAndFeel()
    {
      PlasticLookAndFeel.setMyCurrentTheme(new SkyBlue());
      UIManager.put(Options.USE_SYSTEM_FONTS_APP_KEY, Boolean.TRUE);
      UIManager.put("ClassLoader", LookUtils.class.getClassLoader());
      setLookAndFeel(new Plastic3DLookAndFeel());
    }
  }
}
