/**
 * @author Sandro <sandro.pedrazzini@canoo.com>
 */

package com.canoo.ulc.community.datechooser.application;

import com.ulcjava.base.application.ClientContext;
import com.ulcjava.base.application.ULCComponent;
import com.ulcjava.base.application.util.Color;
import com.ulcjava.base.application.util.Dimension;
import com.ulcjava.base.server.IDispatcher;


import java.text.DateFormat;
import java.util.*;


public class ULCDateChooser extends ULCComponent {

    private Locale fLocale;
    private String fCancelButtonText;
    private Date fSelectedDate;
    private Date fFromDate;
    private Date fToDate;
    private Dimension fDayCellDimension;
    private Color fEnabledCellColor;
    private Color fDisabledCellColor;
    private Color fDialogBackgroundColor;
    // private List fListeners;


    public ULCDateChooser() {
        this(null);
    }

    public ULCDateChooser(Date selectedDate) {
        this(((ClientContext.getLocale() != null) ? ClientContext.getLocale() : Locale.getDefault()), "Cancel", selectedDate);
    }

    public ULCDateChooser(Locale locale, String cancelButton) {
        this(locale, cancelButton, null);
    }

    public ULCDateChooser(Locale locale, String cancelButtonText, Date selectedDate) {
        fLocale = locale;
        fCancelButtonText = cancelButtonText;
        fSelectedDate = selectedDate;
        fDayCellDimension = new Dimension(26, 20);
        fEnabledCellColor = Color.white;
        fDisabledCellColor = new Color(204, 204, 204);
    }

    //set methods

    public void setSelectedDate(Date date) {
        fSelectedDate = (Date) setStateUI("selectedDate", fSelectedDate, date);
    }

    public void setFromDate(Date date) {
        fFromDate = (Date) setStateUI("fromDate", fFromDate, date);
    }

    public void setToDate(Date date) {
        fToDate = (Date) setStateUI("toDate", fToDate, date);
    }

    public void setDayCellDimension(Dimension dimension) {
        fDayCellDimension = (Dimension) setStateUI("dayCellDimension", fDayCellDimension, dimension);
    }

    public void setEnabledCellColor(Color color) {
        fEnabledCellColor = (Color) setStateUI("enabledCellColor", fEnabledCellColor, color);
    }

    public void setDisabledCellColor(Color color) {
        fDisabledCellColor = (Color) setStateUI("disabledCellColor", fDisabledCellColor, color);
    }

    public void setDialogBackgroundColor(Color color) {
        fDialogBackgroundColor = (Color) setStateUI("dialogBackgroundColor", fDialogBackgroundColor, color);
    }

    public void enableCurrentDayView() {
        invokeUI("enableCurrentDayView");
    }

    public void disableCurrentDayView() {
        invokeUI("disableCurrentDayView");
    }

    //get methods

    public Date getSelectedDate() {
        return fSelectedDate;
    }

    public Locale getLocale() {
        return fLocale;
    }

    //utility methods

    public String getSelectedDateAsString(int dateFormat, Date date) {
        try {
            return DateFormat.getDateInstance(dateFormat, getLocale()).format(date);
        } catch (Throwable e) {
            return getSelectedDateAsString();
        }
    }

    public String getSelectedDateAsString(int dateFormat){
        return getSelectedDateAsString(dateFormat,getSelectedDate());
    }

    public String getSelectedDateAsString() {
        return getSelectedDateAsString(getSelectedDate());
    }

    public String getSelectedDateAsString(Date date) {
        return DateFormat.getDateInstance(DateFormat.LONG, getLocale()).format(date);
    }


    // Event handling

    public void addDateSelectedListener(IDateSelectedListener listener) {
        //fListeners.add(listener);
        addListener("dateSelected", listener);
    }

    public void removeDateSelectedListener(IDateSelectedListener listener) {
        removeListener("dateSelected", listener);
    }

    protected DateSelectedEvent createDateSelectedEvent(Date selectedDate) {
        return new DateSelectedEvent(this, selectedDate);
    }

    protected Class getDateSelectedListenerClass() {
        return IDateSelectedListener.class;
    }

    protected void processDateSelectedEvent(String eventCategory, String listenerMethodName, DateSelectedEvent event) {
        updateServerSelectedDate(event.getDate());
        dispatchEvent(eventCategory, listenerMethodName, event);
    }

    protected IDispatcher createDispatcher() {
        return new ULCDateChooserDispatcher();
    }


    protected class ULCDateChooserDispatcher extends ULCComponentDispatcher {
        public final DateSelectedEvent createDateSelectedEvent(Date selectedDate) {
            return ULCDateChooser.this.createDateSelectedEvent(selectedDate);
        }

        public final Class getDateSelectedListenerClass() {
            return ULCDateChooser.this.getDateSelectedListenerClass();
        }

        public final void processDateSelectedEvent(String eventCategory, String listenerMethodName, DateSelectedEvent event) {
            ULCDateChooser.this.processDateSelectedEvent(eventCategory, listenerMethodName, event);
        }
    }

    //overridden methods

    protected String typeString() {
        return "com.canoo.ulc.community.datechooser.client.UIDateChooser";
    }

    protected String getPropertyPrefix() {
        return "DateChooser";
    }

    protected void uploadStateUI() {
        super.uploadStateUI();
        createStateUI(new Object[]{fLocale, fCancelButtonText, fSelectedDate});
        setStateUI("dayCellDimension", null, fDayCellDimension);
        setStateUI("enabledCellColor", null, fEnabledCellColor);
        setStateUI("disabledCellColor", null, fDisabledCellColor);
    }


    private void updateServerSelectedDate(Date date) {
        fSelectedDate = date;
    }
}
