package com.canoo.ulc.community.filetransfer.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import com.ulcjava.base.client.ClientEnvironmentAdapter;
import com.ulcjava.base.client.IFileService;
import com.ulcjava.base.client.ServiceException;
import com.ulcjava.base.client.UIProxy;
import com.ulcjava.base.shared.internal.Anything;

/**
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public class UIFileUploader extends UIProxy {

    private ByteArrayOutputStream uploadedBytes;

    private String clientFilePath;

    private IFileService fileService;

    public UIFileUploader() {
        uploadedBytes = new ByteArrayOutputStream();
    }

    /**
     * @param clientFilePath
     *            The clientFilePath to set.
     */
    public void setClientFilePath(String clientFilePath) {
        this.clientFilePath = clientFilePath;
    }

    protected void preInitializeState() {
        super.preInitializeState();

        fileService = ClientEnvironmentAdapter.getFileService();

        if (fileService == null) {
            sendFailed("File service not available");
        }
    }

    public void handleULCRequest(String request, Anything args) {
        if (request.equals("uploadBytes")) {
            handleUploadBytes(args);
        } else if (request.equals("storeFile")) {
            handleStoreFile();
        } else if (request.equals("uploadCancelled")) {
            handleUploadCancelled();
        } else if (request.equals("uploadFailed")) {
            handleUploadFailed();
        }
    }

    private void handleUploadBytes(Anything args) {
        try {
            uploadedBytes.write((byte[]) args.toObject());
        } catch (IOException e) {
            sendFailed(e.getMessage());
        }
    }

    private void handleStoreFile() {
        ByteArrayInputStream bin = null;
        try {
            bin = new ByteArrayInputStream(uploadedBytes.toByteArray());
            fileService.storeFile(clientFilePath, bin);

            sendUploadComplete();
        } catch (ServiceException e) {
            sendFailed(e.getMessage());
        } catch (NullPointerException ne) {
            sendFailed(ne.getMessage());
        } finally {
            if (bin != null) {
                try {
                    bin.close();
                } catch (IOException ignored) {
                }
            }
        }
    }

    private void handleUploadCancelled() {
        closeUploadedByteStream();
        sendMarkCollectableULC();
    }

    private void handleUploadFailed() {
        closeUploadedByteStream();
        sendMarkCollectableULC();
    }

    private void closeUploadedByteStream() {
        if (uploadedBytes != null) {
            try {
                uploadedBytes.close();
                uploadedBytes = null;
            } catch (IOException ignored) {
            }
        }
    }

    private void sendUploadComplete() {
        closeUploadedByteStream();

        invokeULC("handleUIRequest", new Object[] { "uploadComplete", null });
        sendMarkCollectableULC();
    }

    private void sendFailed(String reason) {
        closeUploadedByteStream();

        invokeULC("handleUIRequest", new Object[] { "uploadFailed", null });
        sendMarkCollectableULC();
    }
}
