package com.canoo.ulc.community.filetransfer.application;

import com.canoo.ulc.community.progresspane.application.TaskControllerAdapter;
import com.canoo.ulc.community.progresspane.application.ITaskList;
import com.canoo.ulc.community.progresspane.application.ITask;
import com.canoo.ulc.community.progresspane.application.AbstractSynchronousTask;
import com.canoo.ulc.community.filetransfer.server.ULCFileDownloader;

/**
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public class FileDownloadTaskList extends TaskControllerAdapter implements ITaskList {
    private long fBytesPerTask;
    private ULCFileDownloader fFileDownloader;
    private boolean fDownloadComplete;
    private boolean fCancelled;

    public FileDownloadTaskList(long maximumFileSizeInBytes, long bytesPerTask, IFileDownloadHandler handler) {
        fBytesPerTask = bytesPerTask;
        fFileDownloader = new ULCFileDownloader(handler, maximumFileSizeInBytes);
        fDownloadComplete = false;
        fCancelled = false;
    }

    public ITask next() {
        return new AbstractSynchronousTask() {
            protected void synchronousExecute() throws Exception {
                if (fCancelled) {
                    fDownloadComplete = true;
                    fFileDownloader.fireDownloadCancelled();
                } else if (fFileDownloader.isFailed()) {
                    fDownloadComplete = true;
                    fFileDownloader.fireDownloadFailed();
                } else if (fFileDownloader.isCompleted()) {
                    fDownloadComplete = true;
                    fFileDownloader.fireDownloadComplete();
                } else {
                    fFileDownloader.downloadNextBytes(fBytesPerTask);
                }
            }

            public String getStatus() {
                return "Uploading " + fFileDownloader.getClientFilePath() + " (" + (fFileDownloader.getDownloadedByteCount() / 1024) + "kB)";
            }
        };
    }

    public boolean hasMore() {
        return !fDownloadComplete;
    }

    public double getPercentCompleted() {
        long downloadSize = fFileDownloader.getDownloadSize();

        if (downloadSize == -1) {
            return fDownloadComplete && !fCancelled ? 1 : 0;
        } else {
            return (double)fFileDownloader.getDownloadedByteCount() / (double)downloadSize;
        }
    }

    public boolean isCancellable() {
        return true;
    }

    public void cancel() {
        fCancelled = true;
    }

    public void error(Exception e) {
        fFileDownloader.fireDownloadFailed(e);
    }

    public void stopped() {
        cancel();
    }
}
