package com.canoo.ulc.community.filetransfer.application;

/**
 * Used to retrieve the path for the client file to be downloaded to the server
 * and to notify on download sucess / failure.
 *
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public interface IFileDownloadHandler {
    /**
     * Returns the path of the file on the client machine to be downloaded to the server.
     *
     * @return the path of the file on the client machine to be downloaded to the server
     */
    public String getClientFilePath();

    /**
     * Callback to indicate that the file download has succeeded. The byte array provided represents
     * the content of the file downloaded from the client to the server.
     *
     * @param downloadedBytes the content of the file downloaded from the client to the server
     */
    public void onSuccess(byte[] downloadedBytes);

    /**
     * Callback to indicate that the file download has failed.
     *
     * @param reason the failure reason
     */
    public void onFailure(Exception reason);

    /**
     * Callback to indicate that the file download has been cancelled (by the user)
     */
    public void onCancel();
}
