package com.canoo.ulc.community.filetransfer.application;

import java.io.InputStream;
import java.io.IOException;

/**
 * Used to retrieve information / data of the file to be uploaded
 * to the client machine and to notify on upload sucess / failure.
 *
 * @author christian.stettler@canoo.com
 * @author Michael Brohl (michael.brohl@agrenon.com) ULC 6.1 port
 */
public interface IFileUploadHandler {
    /**
     * Returns the data to be uploaded to the client machine, made accessible through an input stream.
     *
     * @return an input stream providing access to the data to be uploaded to the client machine
     * @throws IOException if an error during creation / retrieval of the input stream occurs
     */
    public InputStream getInputStream() throws IOException;

    /**
     * Returns the path on the client machine under which the data should be stored as a file. This path could
     * earlier be retrieved using ClientContext.chooseFile().
     *
     * @return the path on the client machine under which the data should be stored
     */
    public String getClientFilePath();

    /**
     * Returns the size of the file / data to be uploaded to the client. If unkown, this method may return -1.
     *
     * @return the size of the file /data to be uploaded to the client, or -1, if unknown
     */
    public long getFileSize();

    /**
     * Callback to indicate that the file upload has succeeded.
     */
    public void onSuccess();

    /**
     * Callback to indicate that the file upload has failed.
     *
     * @param reason the failure reason
     */
    public void onFailure(Exception reason);

    /**
     * Callback to indicate that the file upload has been cancelled (by the user)
     */
    public void onCancel();
}
