/*
 * Copyright  2000-2004 Canoo Engineering AG, Switzerland.
 */

package com.canoo.ulc.community.linksupport.client;

import com.canoo.ulc.community.linksupport.shared.LinkUtilities;
import com.ulcjava.base.client.UITable;
import com.ulcjava.base.shared.IUlcEventConstants;
import com.ulcjava.base.shared.internal.Anything;

import javax.swing.*;
import javax.swing.table.TableCellRenderer;
import java.awt.*;
import java.awt.event.*;

public class UILinkTable extends UITable {
    private static final KeyStroke ENTER_STROKE = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);

    public void restoreState(Anything args) {
        super.restoreState(args);

        JTable table = (JTable)getBasicComponent();
        ActionHandler actionHandler = new ActionHandler(table.getActionForKeyStroke(ENTER_STROKE));
        table.addMouseListener(actionHandler);
        table.addMouseMotionListener(actionHandler);
    }

    private class ActionHandler extends MouseAdapter implements MouseMotionListener, ActionListener {
        private ActionListener fDecoratee;

        public ActionHandler(ActionListener decoratee) {
            fDecoratee = decoratee;
        }

        public void mouseDragged(MouseEvent event) {
        }

        public void mouseMoved(MouseEvent event) {
            if (isOverLink(event.getPoint())) {
                getBasicComponent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            } else {
                getBasicComponent().setCursor(null);
            }
        }

        public void mousePressed(MouseEvent event) {
            if (event.getClickCount() == 1 && SwingUtilities.isLeftMouseButton(event)) {
                if (isOverLink(event.getPoint())) {
                    sendOptionalEventULC(IUlcEventConstants.ACTION_EVENT, IUlcEventConstants.ACTION_PERFORMED);
                }
            }
        }

        public void actionPerformed(ActionEvent event) {
            if (isOptionalEventEnabled(IUlcEventConstants.ACTION_EVENT)) {
                JTable table = (JTable)getBasicComponent();
                if (!table.isEditing()) {
                    sendOptionalEventULC(IUlcEventConstants.ACTION_EVENT, IUlcEventConstants.ACTION_PERFORMED);
                } else {
                    // the original keyboard action stops editing and moves focus to the next row -> we only want to stop editing
                    table.getCellEditor().stopCellEditing();
                }
            } else {
                if (fDecoratee != null) {
                    fDecoratee.actionPerformed(event);
                }
            }
        }

        private boolean isOverLink(Point point) {
            JTable table = (JTable)getBasicComponent();
            int rowAtClick = table.rowAtPoint(point);
            int columnAtClick = table.columnAtPoint(point);

            if (table.isCellEditable(rowAtClick, columnAtClick)) {
                return false;
            }

            TableCellRenderer renderer = table.getCellRenderer(rowAtClick, columnAtClick);
            Component rendererComponent = table.prepareRenderer(renderer, rowAtClick, columnAtClick);
            if (!(rendererComponent instanceof JLabel)) {
                return false;
            }

            JLabel rendererLabel = (JLabel)rendererComponent;
            if (!LinkUtilities.isHtmlLink(rendererLabel.getText())) {
                return false;
            }

            Dimension preferredSize = rendererLabel.getPreferredSize();
            Rectangle clickedRectangle = table.getCellRect(rowAtClick, columnAtClick, false);
            return clickedRectangle.x <= point.x && point.x < clickedRectangle.x + preferredSize.width &&
                    clickedRectangle.y <= point.y && point.y < clickedRectangle.y + preferredSize.height;
        }
    }
}
