/*
 * Copyright  2000-2004 Canoo Engineering AG, Switzerland.
 */
package com.canoo.ulc.community.linksupport.sample.server;

import com.canoo.ulc.community.linksupport.application.ULCLinkLabel;
import com.canoo.ulc.community.linksupport.application.ULCLinkTable;
import com.ulcjava.base.application.*;
import com.ulcjava.base.application.event.ActionEvent;
import com.ulcjava.base.application.event.IActionListener;
import com.ulcjava.base.application.table.AbstractTableModel;
import com.ulcjava.base.application.table.DefaultTableHeaderCellRenderer;
import com.ulcjava.base.application.table.ITableCellRenderer;
import com.ulcjava.base.application.util.Color;
import com.ulcjava.base.application.util.Font;
import com.ulcjava.base.application.util.ULCIcon;

public class ULCLinkSupportSample extends AbstractApplication {
    public void start() {
        ULCLinkLabel linkLabel = new ULCLinkLabel("Label");
        linkLabel.addActionListener(new IActionListener() {
            public void actionPerformed(ActionEvent event) {
                String message = "Label clicked.";
                ULCAlert alert = new ULCAlert("Action", message, "OK");
                alert.setMessageType(ULCAlert.INFORMATION_MESSAGE);
                alert.show();
            }
        });

        final ULCLinkTable linkTable = new ULCLinkTable(new SampleTableModel());
        linkTable.setRowSelectionAllowed(false);
        linkTable.setShowGrid(false);
        linkTable.setRowHeight(linkTable.getRowHeight() + 2);
        linkTable.setDefaultRenderer(Object.class, new ZebraCellRenderer());
        linkTable.getTableHeader().setDefaultRenderer(new HeaderCellRenderer());
        linkTable.getColumnModel().getColumn(0).setCellRenderer(new LinkCellRenderer());
        linkTable.addActionListener(new IActionListener() {
            public void actionPerformed(ActionEvent event) {
                String message = "Table row " + linkTable.getSelectedRow() + " clicked.";
                ULCAlert alert = new ULCAlert("Action", message, "OK");
                alert.setMessageType(ULCAlert.INFORMATION_MESSAGE);
                alert.show();
            }
        });

        ULCBoxPane contentPane = new ULCBoxPane(1, 0);
        contentPane.add(ULCBoxPane.BOX_EXPAND_CENTER, createBorder("Label", linkLabel));
        contentPane.add(ULCBoxPane.BOX_EXPAND_EXPAND, createBorder("Table", linkTable));

        ULCRootPane rootPane = createRootPane();
        rootPane.add(contentPane);
        rootPane.setVisible(true);
    }

    private static ULCBorderLayoutPane createBorder(String title, ULCComponent content) {
    	ULCBorderLayoutPane result = new ULCBorderLayoutPane();
    	result.setBorder(BorderFactory.createTitledBorder(title));
        result.add(content);
        return result;
    }

    protected ULCRootPane createRootPane() {
        ULCFrame result = new ULCFrame("ULCLinkTable Sample");
        result.setIconImage(new ULCIcon(getClass().getResource("application.png")));
        result.setDefaultCloseOperation(ULCFrame.TERMINATE_ON_CLOSE);

        return result;
    }

    private static class SampleTableModel extends AbstractTableModel {
        public Object getValueAt(int row, int column) {
            return row + ":" + column;
        }

        public int getColumnCount() {
            return 5;
        }

        public int getRowCount() {
            return 100;
        }
    }

    private static class ZebraCellRenderer extends ULCLabel implements ITableCellRenderer {
        private Color fEvenBackground;
        private Color fOddBackground;

        public ZebraCellRenderer() {
            fEvenBackground = new Color(222, 231, 239);
            fOddBackground = new Color(189, 206, 222);
        }

        public IRendererComponent getTableCellRendererComponent(ULCTable table, Object value, boolean isSelected, boolean hasFocus, int row) {
            setForeground(table.getForeground());
            setBackground(row % 2 == 0 ? fEvenBackground : fOddBackground);
            setFont(table.getFont());
            return this;
        }
    }

    private static class LinkCellRenderer extends ULCLinkLabel implements ITableCellRenderer {
        private Color fForeground;
        private Color fEvenBackground;
        private Color fOddBackground;

        public LinkCellRenderer() {
            fForeground = new Color(0, 49, 156);
            fEvenBackground = new Color(222, 231, 239);
            fOddBackground = new Color(189, 206, 222);
        }

        public IRendererComponent getTableCellRendererComponent(ULCTable table, Object value, boolean isSelected, boolean hasFocus, int row) {
            setForeground(fForeground);
            setBackground(row % 2 == 0 ? fEvenBackground : fOddBackground);
            setFont(table.getFont());
            return this;
        }
    }

    private static class HeaderCellRenderer extends DefaultTableHeaderCellRenderer {
        public HeaderCellRenderer() {
            setBackground(new Color(189, 206, 222));
            setFont(new Font(getFont().getName(), getFont().getStyle() | Font.BOLD, getFont().getSize()));
        }
    }
}
